/*************************************************************************
 *
 *   Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2008
 *
 *    File name   : drv_nand.c
 *    Description : lpc313x Nand driver
 *
 *    History :
 *    1. Date        : 03.4.2009 . 
 *       Author      : Stoyan Choynev
 *       Description : Initial Revision
 *
 *    $Revision: 30870 $
 **************************************************************************/

/** include files **/

/** local definitions **/
// NAND Flash device commands
#define NAND_RST                    0xFF
#define NAND_READ_A                 0x00
#define NAND_READ_B                 0x01
#define NAND_READ_C                 0x50
#define NAND_READ_ID                0x90
#define NAND_READ_STAT              0x70

#define NAND_PGM_PAGE1              0x80
#define NAND_PGM_PAGE2              0x10

#define NAND_BLOCK_ERASE1           0x60
#define NAND_BLOCK_ERASE2           0xD0

// NAND Flash status register
#define NAND_WRITE_PROT             0x80
#define NAND_ACTIVE                 0x40
#define NAND_ERROR                  0x01
// NAND BASE ADDRESS
#define NAND_RAM_BASE_ADDR          0x70000000

/** default settings **/

/** external functions **/

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/
void InitNAND(void)
{
   /*Select EBI/MPMC pins*/
  SYSCREG_MUX_LCD_EBI_SEL_bit.Mux_LCD_EBI_sel = 1;
  CGU_Run_Clock(EBI_CLK);
  /*Enable MPMC clocks*/
  CGU_Run_Clock(MPMC_CFG_CLK);  
  CGU_Run_Clock(MPMC_CFG_CLK2);
  CGU_Run_Clock(MPMC_CFG_CLK3);

  /* Enable NAND Flash Controller Clocks */
  CGU_Run_Clock(NANDFLASH_S0_CLK);  
  CGU_Run_Clock(NANDFLASH_ECC_CLK);
  CGU_Run_Clock(NANDFLASH_NAND_CLK);  
  CGU_Run_Clock(NANDFLASH_PCLK);
  /*Reset NAND Flash Controller*/
  NANDFLASH_CTRL_ECC_RESET_N_SOFT = 0;
  NANDFLASH_CTRL_NAND_RESET_N_SOFT = 0;
  for(Int32U i = 0 ; 10000 > i; i++);
  NANDFLASH_CTRL_ECC_RESET_N_SOFT = 1;
  NANDFLASH_CTRL_NAND_RESET_N_SOFT = 1;
  /**/
  SYSCREG_MUX_NAND_MCI_SEL_bit.Mux_NAND_MCI_sel = 0;
  
  /*Nand Configuration Register*/
  NandConfig = (1<<0)|     /* Error Correction On    */
               (0<<1)|     /* 8-bit flash device     */
               (1<<3)|     /* DMA enabled           */
               (0<<4)|     /* Little endian          */
               (0<<5)|     /* Latency configuration  */
               (1<<7)|     /* ECC started after 516B */
               (1<<8)|     /* CE deactivated         */
               (0<<10)|    /* 528 Bytes Read/Write   */
               (0<<12)|    /* 5-bit mode ECC         */
               (0<<13)|    /* ECC Clock Gating Off   */
               (0<<15);    /* ECC Clock is On        */
                      
  NandSetCE = 0x1E;
  /* NAND Reset */
  Int32U tmp = NandCheckSTS;
  
  NandSetCmd = NAND_RST;
  // wait for device ready
  while (!(NandCheckSTS_bit.R1R));
 
}

void NandReadID(Int8U * ID)
{
  NandSetCmd = NAND_READ_ID;
  NandSetAddr = 0;
 
  for(Int8U i = 0;4 > i ;i++)
  {
    *ID++ = NandReadData;
  }
 
}
/*************************************************************************
 * Function name: AddrInOperation
 * Parameters:  unsigned int Page
 *
 * Return: None
 *
 * Description: Perform Address Operation
 *
 *************************************************************************/
static void AddrInOperation(Int32U Page)
{
  /*Column Address*/
  NandSetAddr = 0;
  /*Column Address*/
  NandSetAddr = 0;
  /**/
  for(Int32U i = 0 ; 3 > i; i++)
  {
    /*Load Row*/
    NandSetAddr = Page & 0xFF;
    /*Shift Row*/
    Page>>=8;
  }
}

/*************************************************************************
 * Function name: NandGetStatus
 * Parameters:  None
 *
 * Return: flash device status register
 *
 * Description: Reads NAND Flash Status register
 *
 *************************************************************************/
static Int32U NandGetStatus(void)
{
  NandSetCmd = NAND_READ_STAT;
  return NandReadData;
}

__no_init static  Int8U _Buffer[2064] @ 0x11054000;
/*************************************************************************
 * Function Name: NandReadPage
 * Parameters: Page - NAND Flash Page Number. In is Block*NAND_PG_PER_BLK + Page_in_Block
 *
 * Return: FLASH_OK - No Read Error or 1bit Error
 *         FLASH_ERROR - Non Correctable Read Error
 * Description: Reads one page (512 main+6 spare) from NAND Flash into _Buffer.Buffer
 *
 *************************************************************************/
static Int32U NandReadPage(Int32U Page)
{
Int8U * pdata = _Buffer;
  /*Read*/
  NandSetCmd = NAND_READ_A;
  /*Set Address*/
  AddrInOperation(Page);
  
  Int32U tmp = NandCheckSTS;
  
  NandSetCmd = 0x30;
  // wait for device ready
  while (!(NandCheckSTS_bit.R1R));
  /*Read 4*528 bytes*/
  for(Int32U i = 0 ; 4 > i ; i++)
  {
    /*Clear RAM0 decoded values*/
    NandIRQStatusRaw = (1<<11)|(1<<12)|(1<<13)|(1<<14)|(1<<15)|(1<<16)|(1<<17)|(1<<21)|(1<<26);
    /*Read 528 Bytes*/
    NandControlFlow_bit.R0 = 1;
    /**/
    while(!NandIRQStatusRaw_bit.INT21R);
    /**/
   /* if(NandIRQStatusRaw_bit.INT11R && !NandIRQStatusRaw_bit.INT26R)
    {
      return(1);
    }
    */
    Int8U * pnandram = (Int8U *)NAND_RAM_BASE_ADDR;
  
    for (Int32U Counter = 0; Counter < 528; Counter++)
    {
      *pdata++ = *pnandram++;
    }

  }
  return 0;
}
/*************************************************************************
 * Function Name: NandEraseBlock
 * Parameters: Block - NAND Page Number.
 *
 * Return: FLASH_OK - Erase Successful
 *         FLASH_ERROR - Erase Fail
 * Description: Erase One Block
 *
 *************************************************************************/
static Int32U NandEraseBlock(Int32U Page)
{

  /*:TODO*/
  //if((_NAND_Blocks*NAND_PG_PER_BLK) <= Page) return FLASH_ERROR;

  /*Auto Block Erase Setup Command*/
  NandSetCmd = NAND_BLOCK_ERASE1;
  /*Row Address*/
  for(Int32U i = 3; i ; i--)
  {
    /*Load Row*/
    NandSetAddr = Page & 0xFF;
    /*Shift Row*/
     Page>>=8;
  }
  /*Erase Command*/
  Int32U tmp = NandCheckSTS;
  
  NandSetCmd = NAND_BLOCK_ERASE2;
  // wait for device ready
  while (!(NandCheckSTS_bit.R1R));
  if(NandGetStatus() & NAND_ERROR)
  {
    return 1;
  }

  return 0;
}

/*************************************************************************
 * Function Name: NandWritePage
 * Parameters: Page - NAND Flash Page Number. In is Block*NAND_PG_PER_BLK + Page_in_Block
 *             Puff - data for write
 * Return: FLASH_OK - Write Successful
 *         FLASH_ERROR - Write Fail
 * Description: Writes one page (512 main+6 spare) Puff
 *              into NAND Flash
 *************************************************************************/
static Int32U NandWritePage(Int32U Page, const Int32U * pBuff)
{
Int32U Counter;

  NandSetCmd = NAND_PGM_PAGE1;
  /*Set Address*/
  AddrInOperation(Page);

  for(Int32U i = 0; 4 > i ; i++)
  {
    /*Write 2K bytes*/
    /*Clear RAM0 encode flag*/
    //NandIRQStatusRaw = (1<<20);
    Int32U * pNandRam = (Int32U *)(NAND_RAM_BASE_ADDR);
    /*Copy data to NAND controller RAM*/
    NandIRQStatusRaw = (1<<20)|(1<<24); /*Clear Write done flag*/
 
    for (Counter = 0; Counter < (516/4); Counter++)
    {
      *pNandRam++ = *pBuff++;
    }
    
    //while(!NandIRQStatusRaw_bit.INT20R);
    /*Write to Nand*/
    {
     // NandIRQStatusRaw = (1<<24); /*Clear Write done flag*/
      NandControlFlow_bit.W0 = 1; /*Write Start*/
    }
    /*Wait Write to complete*/
    while(!NandIRQStatusRaw_bit.INT24R);

  }

  Int32U tmp = NandCheckSTS;
  
  NandSetCmd = NAND_PGM_PAGE2;
  // wait for device ready
  while (!(NandCheckSTS_bit.R1R));

  /*Get Write operation Status*/
  if(NandGetStatus() & NAND_ERROR)
  {
    return 1;
  }

  return 0;
}

/** private functions **/
